// More information at: https://www.aeq-web.com
// Build: 2019-DEC-20

#include <SPI.h>
#include <LoRa.h>
#include <U8x8lib.h>

#define ss 18
#define rst 14
#define dio0 26

String RxString;

U8X8_SSD1306_128X64_NONAME_SW_I2C u8x8(15, 4, 16);

void setup() {
  Serial.begin(9600);
  u8x8.begin();
  u8x8.setFont(u8x8_font_chroma48medium8_r);
  u8x8.clearDisplay();
  Serial.println("LoRa Rx");
  SPI.begin(5, 19, 27, 18);
  LoRa.setPins(ss, rst, dio0);

  if (!LoRa.begin(868E6)) {
    Serial.println("Starting LoRa failed!");
    while (1);
  }
}

void loop() {
  int packetSize = LoRa.parsePacket();

  if (packetSize) { // If LoRa Packed received
    Serial.println("Receive packet");
    get_lora_data();
    display_status();
  }
}

void get_lora_data() {
  boolean clr = 1;
  while (LoRa.available()) {
    if (clr == 1) {
      RxString = "";
      clr = 0;
    }
    RxString += (char)LoRa.read();
  }
  clr = 1;
}


void display_status() {

  String station_id = split_string( RxString, ';', 0);
  String temp = split_string( RxString, ';', 1);
  String humidity = split_string( RxString, ';', 2);
  String pressure = split_string( RxString, ';', 3);

  u8x8.clearDisplay();
  u8x8.setCursor(0, 1);
  u8x8.print("ST-ID: ");
  u8x8.print(station_id);
  u8x8.setCursor(0, 3);
  u8x8.print("Temp: ");
  u8x8.print(temp);
  u8x8.print("*C");
  u8x8.setCursor(0, 5);
  u8x8.print("Hum.: ");
  u8x8.print(humidity);
  u8x8.print("%");
  u8x8.setCursor(0, 7);
  u8x8.print("Pres.: ");
  u8x8.print(pressure);
  u8x8.print("HPa");

  Serial.print("Station-ID: ");
  Serial.println(station_id);
  Serial.print("Temp: ");
  Serial.print(temp);
  Serial.println(" *C");
  Serial.print("Humidity: ");
  Serial.print(humidity);
  Serial.println(" %");
  Serial.print("Pressure: ");
  Serial.print(pressure);
  Serial.println(" H/Pa");
}

String split_string(String data, char separator, int index)
{
  int found = 0;
  int strIndex[] = { 0, -1 };
  int maxIndex = data.length();

  for (int i = 0; i <= maxIndex && found <= index; i++) {
    if (data.charAt(i) == separator || i == maxIndex) {
      found++;
      strIndex[0] = strIndex[1] + 1;
      strIndex[1] = (i == maxIndex) ? i + 1 : i;
    }
  }
  return found > index ? data.substring(strIndex[0], strIndex[1]) : "";
}
